SystemOrganization addCategory: #'SqueakCheck-Generators'!
SystemOrganization addCategory: #'SqueakCheck-Generators-Tests'!

!Character class methodsFor: '*SqueakCheck-Generators' stamp: 'fbs 1/28/2012 22:01'!
dataFrom: aGenerator
        ^ aGenerator characterData.! !

!ScaledDecimal class methodsFor: '*SqueakCheck-Generators' stamp: 'fbs 1/31/2012 09:21'!
dataFrom: aDataGenerator
	^ self
		newFromNumber: aDataGenerator integerData
		scale: aDataGenerator integerData.! !

Object subclass: #AbstractDataGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakCheck-Generators'!

!AbstractDataGenerator commentStamp: 'fbs 9/9/2011 17:42' prior: 0!
My subclasses provide ways of generating data.

SqueakCheck uses double-dispatch to permit you to easily add your own custom generator, as well as extend the existing protocol to support the generation of different kinds of data.!

!AbstractDataGenerator methodsFor: 'generators' stamp: 'fbs 1/28/2012 20:25'!
anyOf: anArray
	self subclassResponsibility.! !

!AbstractDataGenerator methodsFor: 'generators' stamp: 'fbs 2/18/2012 22:33'!
arrayData
	self subclassResponsibility.! !

!AbstractDataGenerator methodsFor: 'generators' stamp: 'fbs 9/8/2011 20:07'!
booleanData
	self subclassResponsibility! !

!AbstractDataGenerator methodsFor: 'generators' stamp: 'fbs 9/13/2011 21:21'!
characterData
        self subclassResponsibility.! !

!AbstractDataGenerator methodsFor: 'generators' stamp: 'fbs 9/8/2011 20:08'!
integerData
	self subclassResponsibility.! !

!AbstractDataGenerator methodsFor: 'generators' stamp: 'fbs 9/8/2011 21:29'!
objectData
	self subclassResponsibility! !

!AbstractDataGenerator methodsFor: 'generators' stamp: 'fbs 1/31/2012 09:29'!
positiveIntegerData
	| i |
	i := 0.
	[i <= 0] whileTrue: [i := self integerData].
	^ i.! !

!AbstractDataGenerator methodsFor: 'generators' stamp: 'fbs 9/13/2011 21:20'!
stringData
	self subclassResponsibility! !

AbstractDataGenerator subclass: #RandomDataGenerator
	instanceVariableNames: 'rng integerRange'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakCheck-Generators'!

!RandomDataGenerator commentStamp: 'fbs 9/11/2011 16:19' prior: 0!
I generate random (or at least pseudorandom) data for testing theories.

* Boolean data is simply true or false.
* Integer data comes from the range [-integerRange, integerRange].!

!RandomDataGenerator methodsFor: 'generators' stamp: 'fbs 1/29/2012 12:00'!
anyOf: anArray
	| sample |
	sample := anArray atRandom: rng.
	^ (sample isBehavior
		ifTrue: [sample]
		ifFalse: [sample class]) dataFrom: self.! !

!RandomDataGenerator methodsFor: 'generators' stamp: 'fbs 2/20/2012 09:11'!
arrayData
	| any cls |
	any := {[self booleanData]. [self characterData]. [self integerData]. [self stringData]}.
	cls := {Array. OrderedCollection} atRandom: rng.
	^ (1 to: (rng nextInt: 100) + 1)
		inject: cls new
		into: [:arr :unusedInt | arr , {(any atRandom: rng) value}].! !

!RandomDataGenerator methodsFor: 'generators' stamp: 'fbs 9/11/2011 09:52'!
booleanData
	^ #(true false) atRandom: rng.! !

!RandomDataGenerator methodsFor: 'generators' stamp: 'fbs 9/13/2011 21:25'!
characterData
        ^ Character value: (rng nextInt: 65536).! !

!RandomDataGenerator methodsFor: 'initialize-release' stamp: 'fbs 9/11/2011 16:16'!
initialize
	rng := Random new.
	integerRange := 1000.! !

!RandomDataGenerator methodsFor: 'generators' stamp: 'fbs 9/11/2011 16:16'!
integerData
	^ (rng nextInt: (integerRange * 2)) - integerRange! !

!RandomDataGenerator methodsFor: 'configuring' stamp: 'fbs 9/11/2011 16:15'!
integerRange: anInteger
	integerRange := anInteger.! !

!RandomDataGenerator methodsFor: 'generators' stamp: 'fbs 9/27/2011 14:07'!
objectData
	| sel |
	sel := ((self class allSelectors select: [:name | name endsWith: 'Data'])
		reject: [:name | name == #objectData]) atRandom: rng.
	^ self perform: sel.! !

!RandomDataGenerator methodsFor: 'generators' stamp: 'fbs 9/13/2011 21:26'!
stringData
        | stream t |
        t := rng nextInt: integerRange.
        stream := WriteStream on: (WideString new: t).
        t timesRepeat: [stream nextPut: self characterData].
        ^ stream contents.! !

AbstractDataGenerator subclass: #TrivialDataGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakCheck-Generators'!

!TrivialDataGenerator commentStamp: 'fbs 9/14/2011 07:30' prior: 0!
I provide simple constant-value generators. I'm good for checking infrastructure, but you almost certainly don't want to use me to try falsify a real theory.!

!TrivialDataGenerator methodsFor: 'generators' stamp: 'fbs 1/28/2012 20:25'!
anyOf: anArray
	^ anArray first dataFrom: self.! !

!TrivialDataGenerator methodsFor: 'generators' stamp: 'fbs 2/18/2012 22:35'!
arrayData
	^ #(1).! !

!TrivialDataGenerator methodsFor: 'generators' stamp: 'fbs 9/8/2011 20:08'!
booleanData
	^ true.! !

!TrivialDataGenerator methodsFor: 'generators' stamp: 'fbs 9/13/2011 21:21'!
characterData
        ^ $a.! !

!TrivialDataGenerator methodsFor: 'generators' stamp: 'fbs 9/8/2011 20:08'!
integerData
	^ 1.! !

!TrivialDataGenerator methodsFor: 'generators' stamp: 'fbs 9/8/2011 21:29'!
objectData
	^ self.! !

!TrivialDataGenerator methodsFor: 'generators' stamp: 'fbs 9/13/2011 21:20'!
stringData
        ^ 'I am a test string'.! !

!Object class methodsFor: '*SqueakCheck-Generators' stamp: 'fbs 1/28/2012 22:01'!
dataFrom: aDataGenerator 
	^ aDataGenerator objectData! !

!Object methodsFor: '*SqueakCheck-Generators' stamp: 'fbs 1/31/2012 09:37'!
dataFrom: aDataGenerator
	^ self class dataFrom: aDataGenerator.! !

!Complex class methodsFor: '*SqueakCheck-Generators' stamp: 'fbs 1/31/2012 09:20'!
dataFrom: aDataGenerator
	^ self real: aDataGenerator integerData imaginary: aDataGenerator integerData.! !

TestCase subclass: #RandomDataGeneratorTest
	instanceVariableNames: 'generator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakCheck-Generators-Tests'!

!RandomDataGeneratorTest methodsFor: 'running' stamp: 'fbs 9/10/2011 22:26'!
setUp
	generator := RandomDataGenerator new.! !

!RandomDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/28/2012 22:06'!
testACharacterProducesCharacters
        | data |
        data := OrderedCollection new.
        100 timesRepeat: [ | datum |
                datum := Character dataFrom: generator.
                data add: datum.
                self assert: datum isCharacter].
        "Test that the data varies, even though we don't know what that variation looks like."
        self assert: ((data collect: #charCode) range > 0).! !

!RandomDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/29/2012 11:53'!
testAnArrayProducesDataFromItsElements
	| data |
	data := OrderedCollection new.
	10 timesRepeat: [data add: ({true. 1. 'two'. $3.} dataFrom: generator)].
	data := data reject: [:each | {Boolean. SmallInteger. String. Character}
			anySatisfy: [:cls | each isKindOf: cls]].
	self assert: data isEmpty.! !

!RandomDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/28/2012 22:09'!
testAnIntegerProducesIntegers
	| data |
	data := OrderedCollection new.
	100 timesRepeat: [ | datum |
		datum := Integer dataFrom: generator.
		data add: datum.
		self assert: datum isInteger].
	
	"Test that the data varies, even though we don't know what that variation looks like."
	self assert: (data range > 0)! !

!RandomDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 2/18/2012 22:35'!
testArraysProduceArrays
	| data |
	data := OrderedCollection new.
	10 timesRepeat: [data add: (SequenceableCollection dataFrom: generator)].
	data := data reject: [:each | {Array. OrderedCollection}
			anySatisfy: [:cls | each isKindOf: cls]].
	self assert: data isEmpty.! !

!RandomDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/28/2012 22:09'!
testFalseProducesBooleans
	100 timesRepeat: [self assert: (#(true false) includes: (Boolean dataFrom: generator))].! !

!RandomDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/31/2012 09:28'!
testPositiveIntegerDataReturnsPositiveIntegers
	self assert: generator positiveIntegerData > 0.! !

!RandomDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/28/2012 22:10'!
testStringProducesStrings
        | data |
        data := OrderedCollection new.
        100 timesRepeat: [ | datum |
                datum := String dataFrom: generator.
                data add: datum.
                self assert: datum isString].
        "Test that the data varies, even though we don't know what that variation looks like."
        self assert: ((data collect: #size) range > 0)! !

!RandomDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/28/2012 22:10'!
testTrueProducesBooleans
	100 timesRepeat: [self assert: (#(true false) includes: (Boolean dataFrom: generator))]! !

TestCase subclass: #TrivialDataGeneratorTest
	instanceVariableNames: 'generator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakCheck-Generators-Tests'!

!TrivialDataGeneratorTest commentStamp: 'fbs 9/9/2011 08:06' prior: 0!
My methods return constant values. I am thus useful for testing things that need data, but where you don't want random data.!

!TrivialDataGeneratorTest methodsFor: 'running' stamp: 'fbs 9/8/2011 18:29'!
setUp
	generator := TrivialDataGenerator new.! !

!TrivialDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/27/2012 18:58'!
testAnArrayProducesDataFromItsFirstElement
	self assert: true equals: ({true. 1. 'two'. Object new.} dataFrom: generator).! !

!TrivialDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 2/18/2012 22:34'!
testArrayaProduceSingleElementArrays
	self assert: #(1) equals: (SequenceableCollection dataFrom: generator).
	self assert: #(1) equals: (Array dataFrom: generator).
	self assert: #(1) equals: (OrderedCollection dataFrom: generator).! !

!TrivialDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/28/2012 22:11'!
testBooleansProduceTrue
	self assert: true equals: (Boolean dataFrom: generator).! !

!TrivialDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/28/2012 22:12'!
testCharactersProduceA
	self assert: $a equals: (Character dataFrom: generator).! !

!TrivialDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/28/2012 22:12'!
testIntegersProduceOne
	self assert: 1 equals: (Integer dataFrom: generator).! !

!TrivialDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/28/2012 22:19'!
testObjectsProduceSomeConstantObject
	| o |
	o := Object dataFrom: generator.
	{Object new. Class. Smalltalk.} do: [:each |
		self assert: o equals: (each class dataFrom: generator)].! !

!TrivialDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/31/2012 09:28'!
testPositiveIntegerDataReturnsPositiveIntegers
	self assert: generator positiveIntegerData > 0.! !

!TrivialDataGeneratorTest methodsFor: 'testing' stamp: 'fbs 1/28/2012 22:21'!
testStringsProduceaConstantString
	5 timesRepeat: [self assert: 'I am a test string' equals: (String dataFrom: generator)].! !

!Duration class methodsFor: '*SqueakCheck-Generators' stamp: 'fbs 1/31/2012 09:38'!
dataFrom: aDataGenerator
	^ self
		seconds: (aDataGenerator positiveIntegerData)
		nanoSeconds: (aDataGenerator positiveIntegerData).! !

!Boolean class methodsFor: '*SqueakCheck-Generators' stamp: 'fbs 1/28/2012 21:55'!
dataFrom: aDataGenerator 
	^ aDataGenerator booleanData.! !

!SequenceableCollection class methodsFor: '*SqueakCheck-Generators' stamp: 'fbs 2/18/2012 22:33'!
dataFrom: aDataGenerator
	^ aDataGenerator arrayData.! !

!SequenceableCollection methodsFor: '*SqueakCheck-Generators' stamp: 'fbs 1/31/2012 09:36'!
dataFrom: aDataGenerator
	^ self anyOne class dataFrom: aDataGenerator.! !

!Integer class methodsFor: '*SqueakCheck-Generators' stamp: 'fbs 1/28/2012 22:02'!
dataFrom: aDataGenerator
	^ aDataGenerator integerData.! !

!String class methodsFor: '*SqueakCheck-Generators' stamp: 'fbs 1/28/2012 22:02'!
dataFrom: aGenerator
        ^ aGenerator stringData.! !
